@@ -4,18 +4,14 @@ from sklearn.model_selection import KFold
 from photonai.base import Hyperpipe, PipelineElement
 from photonai.optimization import FloatRange, Categorical, IntegerRange
 
-
-# WE USE THE BREAST CANCER SET FROM SKLEARN
-X, y = load_breast_cancer(return_X_y=True)
-
-# DESIGN YOUR PIPELINE
 my_pipe = Hyperpipe('basic_svm_pipe',
                     inner_cv=KFold(n_splits=5),
                     outer_cv=KFold(n_splits=3),
                     optimizer='sk_opt',
-                    optimizer_params={'n_configurations': 25},
+                    optimizer_params={'n_configurations': 15},
                     metrics=['accuracy', 'precision', 'recall', 'balanced_accuracy'],
-                    best_config_metric='accuracy')
+                    best_config_metric='accuracy',
+                    project_folder='./tmp')
 
 my_pipe.add(PipelineElement('StandardScaler'))
 
@@ -28,4 +24,5 @@ my_pipe += PipelineElement('SVC',
                                             'C': FloatRange(1, 6)},
                            gamma='scale')
 
-my_pipe.fit(X, y)
\ No newline at end of file
+X, y = load_breast_cancer(return_X_y=True)
+my_pipe.fit(X, y)
