@@ -1,16 +1,31 @@
 import datetime
 import random
+from typing import Union, Generator
 
 from photonai.optimization.base_optimizer import PhotonSlaveOptimizer
 from photonai.photonlogger.logger import logger
 
 
 class RandomSearchOptimizer(PhotonSlaveOptimizer):
-    """
-     Searches for the best configuration by randomly testing k possible hyperparameter combinations without grid.
-    """
+    """Random search optimizer.
 
-    def __init__(self, n_configurations=None, limit_in_minutes=60):
+    Searches for the best configuration by randomly
+    testing hyperparameter combinations without any grid.
+
+    """
+    def __init__(self, limit_in_minutes: Union[float, None] = 60, n_configurations: Union[int, None] = None):
+        """
+        Initialize the object.
+        One of limit_in_minutes or n_configurations must differ from None.
+
+        Parameters:
+            limit_in_minutes:
+                Total time in minutes.
+
+            n_configurations:
+                Number of configurations to be calculated.
+
+        """
         self.pipeline_elements = None
         self.parameter_iterable = None
         self.ask = self.next_config_generator()
@@ -29,18 +44,37 @@ class RandomSearchOptimizer(PhotonSlaveOptimizer):
             self.n_configurations = n_configurations
         self.k_configutration = 0  # use k++ until k==n: break
 
-        if not n_configurations and limit_in_minutes <= 0:
+        if self.n_configurations is None and self.limit_in_minutes is None:
             msg = "No stopping criteria for RandomSearchOptimizer."
-            logger.warning(msg)
+            logger.error(msg)
+            raise ValueError(msg)
 
-    def prepare(self, pipeline_elements, maximize_metric):
+    def prepare(self, pipeline_elements: list, maximize_metric: bool) -> None:
+        """
+        Initializes grid free random hyperparameter search.
+
+        Parameters:
+            pipeline_elements:
+                List of all PipelineElements to create the hyperparameter space.
+
+            maximize_metric:
+                Boolean to distinguish between score and error.
+
+        """
+        self.start_time = None
         self.pipeline_elements = pipeline_elements
         self.ask = self.next_config_generator()
 
-    def next_config_generator(self):
+    def next_config_generator(self) -> Generator:
+        """
+        Generator for new configs - ask method.
 
+        Returns:
+            Yields the next config.
+
+        """
         while True:
-            val = (yield self.generate_config())
+            _ = (yield self._generate_config())
             self.k_configutration += 1
             if self.limit_in_minutes:
                 if self.start_time is None:
@@ -54,11 +88,7 @@ class RandomSearchOptimizer(PhotonSlaveOptimizer):
                 if self.k_configutration >= self.n_configurations:
                     return
 
-    def tell(self, config, performance):
-        # influence return value of next_config
-        pass
-
-    def generate_config(self):
+    def _generate_config(self):
         config = {}
         for p_element in self.pipeline_elements:
             for h_key, h_value in p_element.hyperparameters.items():
@@ -67,6 +97,3 @@ class RandomSearchOptimizer(PhotonSlaveOptimizer):
                 else:
                     config[h_key] = h_value.get_random_value()
         return config
-
-
-
