@@ -4,26 +4,33 @@ import numpy as np
 
 
 class StratifiedKFoldRegression(_BaseKFold):
-    """Stratified K-Folds cross-validator for continuous target values (regression tasks)
+    """Stratified K-Folds cross-validator for continuous target values (regression tasks).
+
        Provides train/test indices to split data in train/test sets.
+
        This cross-validation object is a variation of StratifiedKFold that returns
        stratified folds with regard to a continuous target. The folds are made by first sorting
        samples with respect to y. Subsequently, we step through each consecutive k samples and
        randomly allocate exactly one of them to one of the test folds. For a detailed description
        of the process see: http://scottclowe.com/2016-03-19-stratified-regression-partitions/.
+
        Read more in the :ref:`User Guide <cross_validation>`.
+
        Parameters
        ----------
        n_splits : int, default=3
            Number of folds. Must be at least 2.
+
        shuffle : boolean, optional
            Whether to shuffle each stratification of the data before splitting
            into batches.
+
        random_state : int, RandomState instance or None, optional, default=None
            If int, random_state is the seed used by the random number generator;
            If RandomState instance, random_state is the random number generator;
            If None, the random number generator is the RandomState instance used
            by `np.random`. Used when ``shuffle`` == True.
+
        Examples
        --------
        >>> from photonai.processing.cross_validation import StratifiedKFoldRegression
@@ -40,6 +47,7 @@ class StratifiedKFoldRegression(_BaseKFold):
        ...    y_train, y_test = y[train_index], y[test_index]
        TRAIN: [1 3 5] TEST: [0 2 4]
        TRAIN: [0 2 4] TEST: [1 3 5]
+
        Notes
        -----
        All the folds have size ``trunc(n_samples / n_splits)``, the last one has
@@ -54,7 +62,8 @@ class StratifiedKFoldRegression(_BaseKFold):
         n_splits = self.n_splits
         y = np.asarray(y)
         if y.ndim > 1:
-            raise NotImplementedError('Target data has more than one dimension. Must be single vector of continuous values.')
+            msg = 'Target data has more than one dimension. Must be single vector of continuous values.'
+            raise NotImplementedError(msg)
         n_samples = y.shape[0]
         self.n_samples = n_samples
         sort_indices = np.argsort(y)
@@ -76,12 +85,12 @@ class StratifiedKFoldRegression(_BaseKFold):
             current = stop
         return test_folds
 
-    def _iter_test_masks(self, X, y, groups=None):
+    def _iter_test_masks(self, X=None, y=None, groups=None):
         test_folds = self._make_test_folds(X, y)
         for i in range(self.n_splits):
             test_mask = np.zeros(self.n_samples, dtype=np.bool)
             test_mask[test_folds[i]] = True
             yield test_mask
 
-    def split(self, X, y, groups=None):
+    def split(self, X=None, y=None, groups=None):
         return super(StratifiedKFoldRegression, self).split(X, y, groups)
