@@ -3,7 +3,7 @@ import pandas as pd
 import dask
 import os
 from dask.distributed import Client
-
+from datetime import timedelta
 from pymodm import connect
 from pymodm.errors import DoesNotExist, ConnectionError
 from pymongo import DESCENDING
@@ -12,7 +12,7 @@ from photonai.base import OutputSettings
 from photonai.photonlogger.logger import logger
 
 from photonai.processing.inner_folds import Scorer
-from photonai.processing.results_structure import MDBPermutationResults, MDBPermutationMetrics, MDBHyperpipe, FoldOperations
+from photonai.processing.results_structure import MDBPermutationResults, MDBPermutationMetrics, MDBHyperpipe
 
 
 class PermutationTest:
@@ -187,9 +187,6 @@ class PermutationTest:
         logger.info("Calculating permutation test results")
         try:
             mother_permutation = PermutationTest.find_reference(mongodb_path, permutation_id)
-            # mother_permutation = MDBHyperpipe.objects.raw({'permutation_id': PermutationTest.get_mother_permutation_id(permutation_id),
-            #                                                'computation_completed': True}).first()
-
         except DoesNotExist:
             return None
         else:
@@ -202,18 +199,15 @@ class PermutationTest:
             if number_of_permutations == 0:
                 number_of_permutations = 1
 
-            true_performances = dict([(m.metric_name, m.value) for m in mother_permutation.metrics_test
-                                      if m.operation == "FoldOperations.MEAN"])
-
+            true_performances = mother_permutation.get_test_metric(operation="mean")
             perm_performances = dict()
             metric_list = list(set([m.metric_name for m in mother_permutation.metrics_test]))
             metrics = PermutationTest.manage_metrics(metric_list, None,
                                                      mother_permutation.hyperpipe_info.best_config_metric)
 
             for _, metric in metrics.items():
-                perm_performances[metric["name"]] = [m.value for i in all_permutations for m in i.metrics_test
-                                                     if m.metric_name == metric["name"]
-                                                     and m.operation == "FoldOperations.MEAN"]
+                perm_performances[metric["name"]] = [i.get_test_metric(metric["name"], operation="mean")
+                                                     for i in all_permutations for m in i.metrics_test]
 
             # Calculate p-value
             p = PermutationTest.calculate_p(true_performance=true_performances, perm_performances=perm_performances,
@@ -311,7 +305,11 @@ class PermutationTest:
         mother_permutation.permutation_id = PermutationTest.get_mother_permutation_id(permutation_id)
         mother_permutation.save()
         result = dict()
-        result["estimated_duration"] = mother_permutation.computation_end_time - mother_permutation.computation_start_time
+        if mother_permutation.computation_end_time is not None and mother_permutation.computation_start_time is not None:
+            result[
+                "estimated_duration"] = mother_permutation.computation_end_time - mother_permutation.computation_start_time
+        else:
+            result["estimated_duration"] = timedelta(seconds=0)
         result["usability"] = PermutationTest.__validate_usability(mother_permutation)
         return result
 
@@ -320,15 +318,12 @@ class PermutationTest:
         if mother_permutation is not None:
             if mother_permutation.dummy_estimator:
                 best_config_metric = mother_permutation.hyperpipe_info.best_config_metric
-                dummy_threshold_to_beat = [i.value for i in mother_permutation.dummy_estimator.test
-                                           if i.metric_name == best_config_metric and i.operation == str(FoldOperations.MEAN)]
-                if len(dummy_threshold_to_beat) > 0:
-                    dummy_threshold_to_beat = dummy_threshold_to_beat[0]
-                    mother_perm_threshold = [i.value for i in mother_permutation.metrics_test
-                                             if i.metric_name == best_config_metric and i.operation == str(FoldOperations.MEAN)]
-                    mother_perm_threshold = mother_perm_threshold[0]
+                dummy_threshold_to_beat = mother_permutation.dummy_estimator.get_test_metric(name=best_config_metric,
+                                                                                             operation="mean")
+                if dummy_threshold_to_beat is not None:
+                    mother_perm_threshold = mother_permutation.get_test_metric(name=best_config_metric,
+                                                                               operation="mean")
                     if mother_permutation.hyperpipe_info.maximize_best_config_metric:
-
                         if mother_perm_threshold > dummy_threshold_to_beat:
                             return True
                         else:
@@ -381,16 +376,3 @@ class PermutationTest:
         else:
             greater_is_better = Scorer.greater_is_better_distinction(metric)
         return greater_is_better
-
-
-
-
-
-
-
-
-
-
-
-
-
