@@ -3,6 +3,7 @@ import os
 import shutil
 import time
 import unittest
+import warnings
 
 import numpy as np
 from sklearn.datasets import load_breast_cancer
@@ -12,13 +13,15 @@ from sklearn.ensemble import RandomForestClassifier
 from sklearn.model_selection import KFold
 from sklearn.pipeline import Pipeline as SKLPipeline
 from sklearn.preprocessing import StandardScaler
+from sklearn.inspection import permutation_importance
 from keras.metrics import Accuracy
 
 from photonai.base import PipelineElement, Hyperpipe, OutputSettings, Preprocessing, CallbackElement, Branch, Stack, \
     Switch, ParallelBranch
 from photonai.optimization import IntegerRange, Categorical
+from photonai.optimization.optimization_info import Optimization
 from photonai.processing.results_handler import ResultsHandler
-from photonai.processing.results_structure import MDBConfig, MDBFoldMetric, FoldOperations, \
+from photonai.processing.results_structure import MDBConfig, MDBFoldMetric, \
     MDBInnerFold, MDBOuterFold, MDBScoreInformation, MDBDummyResults, MDBHyperpipe
 from photonai.helper.dummy_elements import DummyTransformer, DummyYAndCovariatesTransformer
 from photonai.helper.photon_base_test import elements_to_dict, PhotonBaseTest
@@ -27,13 +30,12 @@ from photonai.helper.photon_base_test import elements_to_dict, PhotonBaseTest
 class HyperpipeTests(PhotonBaseTest):
 
     def setup_hyperpipe(self, output_settings=None):
-        if output_settings is None:
-            output_settings = OutputSettings(project_folder=self.tmp_folder_path)
         self.hyperpipe = Hyperpipe('god', inner_cv=self.inner_cv_object,
                                    metrics=self.metrics,
                                    best_config_metric=self.best_config_metric,
+                                   project_folder=self.tmp_folder_path,
                                    output_settings=output_settings,
-                                   verbosity=2)
+                                   verbosity=0)
         self.hyperpipe += self.ss_pipe_element
         self.hyperpipe += self.pca_pipe_element
         self.hyperpipe.add(self.svc_pipe_element)
@@ -69,7 +71,7 @@ class HyperpipeTests(PhotonBaseTest):
         self.assertIsNotNone(self.hyperpipe.cross_validation)
         self.assertEqual(self.hyperpipe.cross_validation.inner_cv, self.inner_cv_object)
         self.assertIsNone(self.hyperpipe.cross_validation.outer_cv, None)
-        self.assertTrue(self.hyperpipe.cross_validation.eval_final_performance)
+        self.assertTrue(self.hyperpipe.cross_validation.use_test_set)
         self.assertTrue(self.hyperpipe.cross_validation.calculate_metrics_per_fold)
         self.assertFalse(self.hyperpipe.cross_validation.calculate_metrics_across_folds)
         self.assertIsNone(self.hyperpipe.cross_validation.outer_folds)
@@ -113,24 +115,28 @@ class HyperpipeTests(PhotonBaseTest):
     def test_sanity(self):
         # make sure that no metrics means raising an error
         with self.assertRaises(ValueError):
-            hyperpipe = Hyperpipe("hp_name", inner_cv=self.inner_cv_object)
+            Hyperpipe("hp_name", inner_cv=self.inner_cv_object)
 
         # make sure that if no best config metric is given, PHOTON raises a warning
-        with self.assertRaises(Warning):
-            hyperpipe = Hyperpipe("hp_name", inner_cv=self.inner_cv_object, metrics=["accuracy", "f1_score"])
+        # with self.assertWarns(Warning) as w:
+        # with warnings.catch_warnings(record=True) as w:
+        with self.assertRaises(Warning) as w:
+            Hyperpipe("hp_name", inner_cv=self.inner_cv_object, metrics=["accuracy", "f1_score"])
+            assert any("No best config metric was given" in s for s in [e.message.args[0] for e in w])
 
-        with self.assertRaises(Warning):
-            hyperpipe = Hyperpipe("hp_name", inner_cv=self.inner_cv_object, best_config_metric=["accuracy", "f1_score"])
+        # with warnings.catch_warnings(record=True) as w:
+        with self.assertRaises(Warning) as w:
+            Hyperpipe("hp_name", inner_cv=self.inner_cv_object, best_config_metric=["accuracy", "f1_score"])
+            assert any("Best Config Metric must be a single" in s for s in [e.message.args[0] for e in w])
 
         with self.assertRaises(NotImplementedError):
-            hyperpipe = Hyperpipe("hp_name", inner_cv=self.inner_cv_object,
-                                  best_config_metric='accuracy', metrics=["accuracy"],
-                                  calculate_metrics_across_folds=False,
-                                  calculate_metrics_per_fold=False)
+            Hyperpipe("hp_name", inner_cv=self.inner_cv_object,
+                                 best_config_metric='accuracy', metrics=["accuracy"],
+                                 calculate_metrics_across_folds=False,
+                                 calculate_metrics_per_fold=False)
 
         with self.assertRaises(AttributeError):
-            hyperpipe = Hyperpipe("hp_name",
-                                  best_config_metric='accuracy', metrics=["accuracy"])
+            Hyperpipe("hp_name", best_config_metric='accuracy', metrics=["accuracy"])
 
         data = np.random.random((500, 50))
 
@@ -146,7 +152,7 @@ class HyperpipeTests(PhotonBaseTest):
         self.hyperpipe = Hyperpipe('god', inner_cv=self.inner_cv_object,
                                    metrics=[('custom_metric', custom_metric), 'accuracy'],
                                    best_config_metric=Accuracy,
-                                   output_settings=OutputSettings(project_folder=self.tmp_folder_path))
+                                   project_folder=self.tmp_folder_path)
         self.hyperpipe += self.ss_pipe_element
         self.hyperpipe.add(self.svc_pipe_element)
         self.hyperpipe.fit(self.__X, self.__y)
@@ -159,8 +165,8 @@ class HyperpipeTests(PhotonBaseTest):
         self.assertEqual(expected_num_of_metrics, 3)
 
         # dummy average values
-        self.assertTrue(len(self.hyperpipe.results.dummy_estimator.train), expected_num_of_metrics)
-        self.assertTrue(len(self.hyperpipe.results.dummy_estimator.test), expected_num_of_metrics)
+        self.assertTrue(len(self.hyperpipe.results.dummy_estimator.metrics_train), expected_num_of_metrics)
+        self.assertTrue(len(self.hyperpipe.results.dummy_estimator.metrics_test), expected_num_of_metrics)
 
         # overall average values
         self.assertTrue(len(self.hyperpipe.results.metrics_train), 2 * expected_num_of_metrics)
@@ -176,6 +182,77 @@ class HyperpipeTests(PhotonBaseTest):
 
         self.assertTrue(np.array_equal(self.__y + 1, self.hyperpipe.data.y))
 
+    def test_permutation_feature_importances(self):
+        hp = Hyperpipe('god',
+                       outer_cv=KFold(n_splits=3),
+                       inner_cv=KFold(n_splits=2),
+                       metrics=self.metrics,
+                       best_config_metric=self.best_config_metric,
+                       project_folder=self.tmp_folder_path,
+                       verbosity=0)
+        svc = PipelineElement('SVC')
+        hp += svc
+        hp.fit(self.__X, self.__y)
+
+        score_photon = hp.score(self.__X, self.__y)
+        score_element = svc.score(self.__X, self.__y)
+        self.assertAlmostEqual(score_photon, score_element)
+
+        # do it on outer folds
+        permutation_list_outer = hp._calculate_permutation_importances(n_repeats=5, random_state=0)
+        self.assertEqual(len(permutation_list_outer["mean"]), 3)
+
+        permutation_score_outer = hp.get_permutation_feature_importances(n_repeats=5, random_state=0)
+        self.assertTrue("mean" in permutation_score_outer)
+        self.assertTrue("std" in permutation_score_outer)
+        self.assertEqual(permutation_score_outer["mean"].shape, (self.__X.shape[1],))
+        self.assertEqual(permutation_score_outer["std"].shape, (self.__X.shape[1],))
+
+        # do it on inner folds but on training sets from outer split
+        hp.cross_validation.use_test_set = False
+        hp.fit(self.__X, self.__y)
+        permutation_list_inner = hp._calculate_permutation_importances(n_repeats=5)
+        self.assertEqual(len(permutation_list_inner["mean"]), 3*2)
+        permutation_score_inner = hp.get_permutation_feature_importances(n_repeats=5)
+        self.assertEqual(permutation_score_inner["mean"].shape, (self.__X.shape[1],))
+        self.assertEqual(permutation_score_inner["std"].shape, (self.__X.shape[1],))
+        # check that validation set permutation importances (inner folds) differ from those of test set (outer folds)
+        self.assertFalse(np.array_equal(permutation_score_outer["mean"], permutation_score_inner["mean"]))
+
+        # do it on inner folds only
+        hp.cross_validation.outer_folds = {}
+        hp.cross_validation.outer_cv = None
+        hp.cross_validation.use_test_set = False
+        hp.fit(self.__X, self.__y)
+        permutation_list_no_outer = hp._calculate_permutation_importances(n_repeats=5)
+        self.assertEqual(len(permutation_list_no_outer), 2)
+        permutation_score_no_outer = hp.get_permutation_feature_importances(n_repeats=5)
+        self.assertEqual(permutation_score_inner["mean"].shape, (self.__X.shape[1],))
+        self.assertEqual(permutation_score_inner["std"].shape, (self.__X.shape[1],))
+
+        # raise error
+        def fake_metric(y_true, y_pred):
+           return 'a'
+
+        hp = Hyperpipe('god',
+                       outer_cv=KFold(n_splits=3),
+                       inner_cv=KFold(n_splits=2),
+                       metrics=[('fake_metric', fake_metric)],
+                       best_config_metric='fake_metric',
+                       project_folder=self.tmp_folder_path,
+                       verbosity=0)
+        svc = PipelineElement('SVC')
+        hp += svc
+        try:
+            hp.fit(self.__X, self.__y)
+        except Exception as e:
+            # should produce an error so that hp.results.best_config is None.
+            pass
+        with self.assertRaises(ValueError):
+            hp.get_permutation_feature_importances(n_repeats=5)
+        with self.assertRaises(ValueError):
+            hp._calculate_permutation_importances(n_repeats=5)
+
     def test_estimation_type(self):
         def callback(X, y=None, **kwargs):
             pass
@@ -222,7 +299,7 @@ class HyperpipeTests(PhotonBaseTest):
 
     def test_save_optimum_pipe(self):
         tmp_path = os.path.join(self.tmp_folder_path, 'optimum_pipypipe')
-        settings = OutputSettings(project_folder=tmp_path, overwrite_results=True)
+        settings = OutputSettings(overwrite_results=True)
 
         my_pipe = Hyperpipe('hyperpipe',
                             optimizer='random_grid_search',
@@ -231,7 +308,8 @@ class HyperpipeTests(PhotonBaseTest):
                             best_config_metric='f1_score',
                             outer_cv=KFold(n_splits=2),
                             inner_cv=KFold(n_splits=2),
-                            verbosity=1,
+                            verbosity=0,
+                            project_folder=tmp_path,
                             output_settings=settings)
 
         preproc = Preprocessing()
@@ -282,7 +360,7 @@ class HyperpipeTests(PhotonBaseTest):
 
     def test_save_optimum_pipe_custom_element(self):
         tmp_path = os.path.join(self.tmp_folder_path, 'optimum_pipypipe')
-        settings = OutputSettings(project_folder=tmp_path, overwrite_results=True)
+        settings = OutputSettings(overwrite_results=True)
 
         my_pipe = Hyperpipe('hyperpipe',
                             optimizer='random_grid_search',
@@ -291,7 +369,8 @@ class HyperpipeTests(PhotonBaseTest):
                             best_config_metric='f1_score',
                             outer_cv=KFold(n_splits=2),
                             inner_cv=KFold(n_splits=2),
-                            verbosity=1,
+                            verbosity=0,
+                            project_folder=tmp_path,
                             output_settings=settings)
         my_pipe += PipelineElement('KerasDnnClassifier', {}, epochs=1, hidden_layer_sizes=[5])
         my_pipe.fit(self.__X, self.__y)
@@ -305,7 +384,7 @@ class HyperpipeTests(PhotonBaseTest):
 
     def test_failure_to_save_optimum_pipe(self):
         tmp_path = os.path.join(self.tmp_folder_path, 'optimum_pipypipe')
-        settings = OutputSettings(project_folder=tmp_path, overwrite_results=True)
+        settings = OutputSettings(overwrite_results=True)
 
         my_pipe = Hyperpipe('hyperpipe',
                             optimizer='random_grid_search',
@@ -314,7 +393,8 @@ class HyperpipeTests(PhotonBaseTest):
                             best_config_metric='f1_score',
                             outer_cv=KFold(n_splits=2),
                             inner_cv=KFold(n_splits=2),
-                            verbosity=1,
+                            verbosity=0,
+                            project_folder=tmp_path,
                             output_settings=settings)
         my_pipe += PipelineElement('KNeighborsClassifier')
         my_pipe.fit(self.__X, self.__y)
@@ -330,8 +410,7 @@ class HyperpipeTests(PhotonBaseTest):
             return os.path.join(self.hyperpipe.output_settings.results_folder, 'photon_summary.txt')
 
         # Case 1: default
-        output_settings1 = OutputSettings(project_folder=self.tmp_folder_path,
-                                          save_output=True,
+        output_settings1 = OutputSettings(save_output=True,
                                           overwrite_results=False)
         self.setup_hyperpipe(output_settings1)
         self.hyperpipe.fit(self.__X, self.__y)
@@ -348,8 +427,7 @@ class HyperpipeTests(PhotonBaseTest):
         self.assertNotEqual(tmp_path, tmp_path2)
 
         # Case 2 overwrite results: all in the same folder
-        output_settings2 = OutputSettings(project_folder=self.tmp_folder_path,
-                                          save_output=True,
+        output_settings2 = OutputSettings(save_output=True,
                                           overwrite_results=True)
         self.setup_hyperpipe(output_settings2)
         self.hyperpipe.fit(self.__X, self.__y)
@@ -411,7 +489,7 @@ class HyperpipeTests(PhotonBaseTest):
         nmb_list = list()
         for i in range(5):
             nmb = ParallelBranch(name=str(i), nr_of_processes=i+3)
-            sp = PipelineElement('PCA', hyperparameters= {'n_components': IntegerRange(1, 50)})
+            sp = PipelineElement('PCA', hyperparameters={'n_components': IntegerRange(1, 50)})
             nmb += sp
             nmb_list.append(nmb)
 
@@ -432,6 +510,10 @@ class HyperpipeTests(PhotonBaseTest):
         self.hyperpipe.add(PipelineElement('SVC'))
         return nmb_list
 
+    def test_eval_final_performance_deprecation(self):
+        with self.assertRaises(DeprecationWarning):
+            Hyperpipe('name', eval_final_performance=True)
+
     def test_recursive_disabling(self):
         list_of_elements_to_detect = self.setup_crazy_pipe()
         self.hyperpipe._pipe = Branch.prepare_photon_pipe(list_of_elements_to_detect)
@@ -450,8 +532,26 @@ class HyperpipeTests(PhotonBaseTest):
             expected_folder = os.path.join(start_folder, nmbranch.name)
             self.assertEqual(nmbranch.base_element.cache_folder, expected_folder)
 
+    def test_setup_error_file(self):
+        # when we init the hyperpipe the file should exist
+        self.assertTrue(os.path.isfile(self.hyperpipe.output_settings.setup_error_file))
+        # when we call fit it shall disappear
+        # we call it empty so that the computation does not occur
+        with self.assertRaises(ValueError):
+            self.hyperpipe.fit(None, None)
+        # however the file should be gone by now
+        self.assertFalse(os.path.isfile(self.hyperpipe.output_settings.setup_error_file))
+
+
     def test_prepare_result_logging(self):
         # test that results object is given and entails hyperpipe infos
+        rfc = PipelineElement('RandomForestClassifier')
+        lsvc = PipelineElement('LinearSVC')
+        branch = Branch('dummy_branch')
+        branch += PipelineElement('SVC')
+        self.hyperpipe += Stack('final_stack', [PipelineElement('SVC'), rfc, branch])
+        self.hyperpipe += lsvc
+
         self.hyperpipe.data.X = self.__X
         self.hyperpipe.data.y = self.__y
         self.hyperpipe._prepare_result_logging(datetime.datetime.now())
@@ -459,13 +559,25 @@ class HyperpipeTests(PhotonBaseTest):
         self.assertTrue(isinstance(self.hyperpipe.results_handler, ResultsHandler))
         self.assertTrue(len(self.hyperpipe.results.outer_folds) == 0)
 
+        expected_pipeline_struct = {'StandardScaler': str(type(self.ss_pipe_element.base_element)),
+                                    'PCA': str(type(self.pca_pipe_element.base_element)),
+                                    'SVC': str(type(self.svc_pipe_element.base_element)),
+                                    'STACK:final_stack': {'SVC': str(type(self.svc_pipe_element.base_element)),
+                                                          'RandomForestClassifier': str(type(rfc.base_element)),
+                                                          'BRANCH:dummy_branch': {
+                                                              'SVC': str(type(self.svc_pipe_element.base_element))
+                                                          }},
+                                    'LinearSVC': str(type(lsvc.base_element))
+                                    }
+        self.assertDictEqual(self.hyperpipe.results.hyperpipe_info.elements, expected_pipeline_struct)
+
     def test_finalize_optimization(self):
         # it is kind of difficult to test that's why we fake it
         self.hyperpipe.fit(self.__X, self.__y)
 
         # reset all infos
-        self.hyperpipe.results.dummy_estimator.train = MDBScoreInformation()
-        self.hyperpipe.results.dummy_estimator.test = MDBScoreInformation()
+        self.hyperpipe.results.dummy_estimator.metrics_train = MDBScoreInformation()
+        self.hyperpipe.results.dummy_estimator.metrics_test = MDBScoreInformation()
         self.hyperpipe.results.metrics_train = {}
         self.hyperpipe.results.metrics_test = {}
         self.hyperpipe.best_config = None
@@ -477,8 +589,8 @@ class HyperpipeTests(PhotonBaseTest):
 
         expected_num_of_metrics = len(self.hyperpipe.optimization.metrics)
         # dummy average values
-        self.assertTrue(len(self.hyperpipe.results.dummy_estimator.train), expected_num_of_metrics)
-        self.assertTrue(len(self.hyperpipe.results.dummy_estimator.test), expected_num_of_metrics)
+        self.assertTrue(len(self.hyperpipe.results.dummy_estimator.metrics_train), expected_num_of_metrics)
+        self.assertTrue(len(self.hyperpipe.results.dummy_estimator.metrics_test), expected_num_of_metrics)
         # overall average values
         self.assertTrue(len(self.hyperpipe.results.metrics_train), 2 * expected_num_of_metrics)
         self.assertTrue(len(self.hyperpipe.results.metrics_test), 2 * expected_num_of_metrics)
@@ -502,6 +614,32 @@ class HyperpipeTests(PhotonBaseTest):
         loaded_array = np.loadtxt(open(backmapped_feature_importances, 'rb'), delimiter=",")
         self.assertEqual(loaded_array.shape[0], self.__X.shape[1])
 
+    def test_finalize_optimization_preprocessing(self):
+        self.hyperpipe.elements = list()
+
+        pre_proc = Preprocessing()
+        pre_proc += PipelineElement('StandardScaler')
+        self.hyperpipe.add(pre_proc)
+        self.hyperpipe.add(PipelineElement('SVC'))
+        self.hyperpipe.fit(self.__X, self.__y)
+
+        self.assertTrue(os.path.isfile(os.path.join(self.hyperpipe.output_settings.results_folder,
+                                                    'photon_best_model.photon')))
+
+    def test_finalize_optimization_preprocessing_with_client(self):
+        self.hyperpipe.elements = list()
+
+        pb = ParallelBranch(name="ParallelBranch", nr_of_processes=2)
+        pb += PipelineElement('LabelEncoder')
+        pre_proc = Preprocessing()
+        pre_proc += pb
+        self.hyperpipe.add(pre_proc)
+        self.hyperpipe.add(PipelineElement('SVC'))
+        self.hyperpipe.fit(self.__X, self.__y)
+
+        self.assertTrue(os.path.isfile(os.path.join(self.hyperpipe.output_settings.results_folder,
+                                                    'photon_best_model.photon')))
+
     def test_optimum_pipe_predict_and_predict_proba_and_transform(self):
         # find best config and test against sklearn
         self.hyperpipe.elements[-1] = PipelineElement('RandomForestClassifier', {'n_estimators': IntegerRange(4, 20,
@@ -533,24 +671,24 @@ class HyperpipeTests(PhotonBaseTest):
             step2 = self.sklearn_pipe.named_steps["PCA"].transform(self.__X)
         else:
             step2 = step1
-        self.assertTrue(np.array_equal(step2, self.hyperpipe.transform(self.__X)))
+        self.assertTrue(np.allclose(step2, self.hyperpipe.transform(self.__X)))
 
 
 class HyperpipeOptimizationClassTests(unittest.TestCase):
 
     def test_best_config_metric(self):
-        my_pipe_optimizer = Hyperpipe.Optimization('grid_search', {}, [], 'balanced_accuracy', None)
+        my_pipe_optimizer = Optimization('grid_search', {}, [], 'balanced_accuracy', None)
         self.assertTrue(my_pipe_optimizer.maximize_metric)
-        my_pipe_optimizer = Hyperpipe.Optimization('grid_search', {}, [], 'mean_squared_error', None)
+        my_pipe_optimizer = Optimization('grid_search', {}, [], 'mean_squared_error', None)
         self.assertFalse(my_pipe_optimizer.maximize_metric)
 
     def test_optmizer_input_str(self):
         with self.assertRaises(ValueError):
-            my_pipe_optimizer = Hyperpipe.Optimization('unknown_optimizer', {}, [], 'accuracy', None)
+            my_pipe_optimizer = Optimization('unknown_optimizer', {}, [], 'accuracy', None)
 
-        for name, opt_class in Hyperpipe.Optimization.OPTIMIZER_DICTIONARY.items():
-            def get_optimizer(name):
-                my_pipe_optimizer = Hyperpipe.Optimization(name, {}, [], 'accuracy', None)
+        for name, opt_class in Optimization.OPTIMIZER_DICTIONARY.items():
+            def get_optimizer(name, params={}):
+                my_pipe_optimizer = Optimization(name, params, [], 'accuracy', None)
                 return my_pipe_optimizer.get_optimizer()
 
             if name == 'smac':
@@ -559,37 +697,57 @@ class HyperpipeOptimizationClassTests(unittest.TestCase):
                 except ModuleNotFoundError:
                     with self.assertRaises(ModuleNotFoundError):
                         get_optimizer(name)
+            if name =='switch':
+                get_optimizer(name, {'name': 'random_grid_search'})
             else:
                 self.assertIsInstance(get_optimizer(name), opt_class)
 
     def test_get_optimum_config(self):
-        my_pipe_optimizer = Hyperpipe.Optimization('grid_search', {}, [], 'balanced_accuracy', None)
+        my_pipe_optimizer = Optimization('grid_search', {}, [], 'balanced_accuracy', None)
         list_of_tested_configs = list()
         metric_default = MDBFoldMetric(metric_name='balanced_accuracy',
-                                       operation=FoldOperations.MEAN,
+                                       operation="mean",
                                        value=0.5)
         metric_best = MDBFoldMetric(metric_name='balanced_accuracy',
-                                    operation=FoldOperations.MEAN,
+                                    operation="mean",
                                     value=0.99)
+        metric_filter_best = MDBFoldMetric(metric_name='another_accuracy',
+                                           operation="mean",
+                                           value=0.75)
+        metric_filter_default = MDBFoldMetric(metric_name='another_accuracy',
+                                              operation="mean",
+                                              value=0.55)
         # we add looser configs, one good config, and one good config that failed
         # and check if the good non-failing config is chosen
         for i in range(10):
             config = MDBConfig()
             # number 5 is the winner
             if i == 5 or i == 8:
-                config.metrics_test = [metric_best]
+                config.metrics_test = [metric_best, metric_filter_default]
+                config.config_dict = {'any_key': 'pressed'}
+            elif i == 7:
+                config.metrics_test = [metric_default, metric_filter_best]
+                config.config_dict = {'any_key': 'pressed'}
             else:
-                config.metrics_test = [metric_default]
+                config.metrics_test = [metric_default, metric_filter_default]
+                config.config_dict = {'any_key': 'not_pressed'}
             if i == 8:
                 config.config_failed = True
             list_of_tested_configs.append(config)
 
-        winner_config = my_pipe_optimizer.get_optimum_config(list_of_tested_configs)
+        outer_fold = MDBOuterFold()
+        outer_fold.tested_config_list = list_of_tested_configs
+        winner_config = outer_fold.get_optimum_config(metric=my_pipe_optimizer.best_config_metric,
+                                                      maximize_metric=my_pipe_optimizer.maximize_metric)
         self.assertIs(winner_config, list_of_tested_configs[5])
-        self.assertEqual(winner_config.metrics_test[0].value, 0.99)
+        self.assertEqual(winner_config.get_test_metric('balanced_accuracy'), 0.99)
+
+        winner_config_filtered = outer_fold.get_optimum_config(metric='another_accuracy', maximize_metric=True,
+                                                               dict_filter=('any_key', 'pressed'))
+        self.assertEqual(winner_config_filtered.get_test_metric('another_accuracy'), 0.75)
 
     def test_get_optimum_config_outer_folds(self):
-        my_pipe_optimizer = Hyperpipe.Optimization('grid_search', {}, [], 'balanced_accuracy', None)
+        my_pipe_optimizer = Optimization('grid_search', {}, [], 'balanced_accuracy', None)
 
         outer_fold_list = list()
         for i in range(10):
